#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Perform Sliding Window Extraction for Candidate Synonym Swapping """


from typing import Callable

from baseblock import Stopwatch
from baseblock import BaseObject
from baseblock import TextMatcher
from baseblock import TextUtils

from fast_sentence_tokenize import Tokenizer


class PerformTextReplacement(BaseObject):
    """ Perform Sliding Window Extraction for Candidate Synonym Swapping """

    def __init__(self,
                 original_text: str,
                 find_canon: Callable):
        """ Change Log:

        Created:
            26-Aug-2022
            craigtrim@gmail.com
            *   refactor sliding window algorithm
                https://github.com/craigtrim/dbpedia-tag/issues/2
        """
        BaseObject.__init__(self, __name__)
        self._tokenize = Tokenizer().input_text
        self._find_canon = find_canon
        self._original_text = original_text
        self._d_map = {}

    def map(self) -> dict:
        return self._d_map

    def _process(self,
                 input_text: str,
                 d_entity_map: dict) -> dict:

        for original_entity in d_entity_map:

            canon = d_entity_map[original_entity]

            # ensure we are dealing with a true canonical form
            # and not the variant form ...
            canon = self._find_canon(canon)

            key = f"entity_{canon.replace(' ', '_').lower()}"

            x, y = TextMatcher.coords(value=original_entity,
                                      input_text=input_text,
                                      case_sensitive=False)

            if x is None or y is None:
                continue

            self._d_map[key] = {
                'canon': canon,
                'coords': [x, y],
                'original': original_entity
            }

            input_text = TextMatcher.replace(input_text=input_text,
                                             old_value=original_entity,
                                             new_value=key,
                                             case_sensitive=False,
                                             recursive=False)

        return input_text

    def process(self,
                d_entity_map: dict,
                input_text: str) -> str:
        """ Entry Point

        Args:
            d_entity_map (dict): maps original form to canonical form
                e.g., 
                {
                    'a&w restaurant': 'a&w_restaurants'
                }
            input_text (str): the input text in which original forms will be replaced with canonical forms
                e.g.,       "let's go to the a&w restaurant"
                becomes     "let's go to the entity_a&w_restaurants"          

        Returns:
            str: the modified text
        """

        sw = Stopwatch()

        output_text = self._process(input_text=input_text,
                                    d_entity_map=d_entity_map)

        if self.isEnabledForInfo:
            self.logger.info('\n'.join([
                "Text Replacement Completed",
                f"\tInput Text: {input_text}",
                f"\tOutput Text: {output_text}",
                f"\tTotal Time: {str(sw)}"]))

        return output_text




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Orchestrate dbPedia Filtering of Input Entities """


from dbpedia_tag.bp import Tagger
import logging

components = [
    'dbpedia_tag.dmo.candidate_fuzzy_match',
    'dbpedia_tag.dmo.sliding_window_blacklist',
    'dbpedia_tag.dmo.sliding_window_validate',
    'dbpedia_tag.dmo.sliding_window_match',
    'dbpedia_tag.svc.find_exact_matches',
    'dbpedia_tag.svc.perform_text_replacement',
]

[logging.getLogger(x).setLevel(logging.INFO) for x in components]


tagger = Tagger().process


def filter_entity_list(entities: list) -> dict:

    d_tags = {}
    for entity in entities:
        d_result = tagger(entity)
        d_map = d_result['events'][0]['data']['map']

        d_tags[entity] = [k for k in d_map]

    return d_tags



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Blacklist certain Tokens from the sliding window result """


from baseblock import EnvIO
from baseblock import TextUtils
from baseblock import Enforcer
from baseblock import Stopwatch
from baseblock import BaseObject

from dbpedia_tag.dto import stopwords


MIN_LEN_THRESHOLD = EnvIO.int_or_default('MIN_LEN_THRESHOLD', 4)


class SlidingWindowBlacklist(BaseObject):
    """ Blacklist certain Tokens from the sliding window result """

    def __init__(self):
        """
        Created:
            24-Aug-2022
            craigtrim@gmail.com
        Updated:
            26-Aug-2022
            craigtrim@gmail.com
            *   refactor filtering into multiple methods
        """
        BaseObject.__init__(self, __name__)

    def _length_check(self,
                      candidates: list) -> list:

        normalized = []
        for candidate in candidates:

            len_of_candidate = len(candidate)
            if len_of_candidate < MIN_LEN_THRESHOLD:
                continue

            # count the length minus the spaces
            if len_of_candidate - candidate.count(' ') < MIN_LEN_THRESHOLD:
                continue

            normalized.append(candidate)

        return normalized

    def _punkt_check(self,
                     candidates: list) -> list:

        normalized = []
        for candidate in candidates:

            if ' ' not in candidate:
                normalized.append(candidate)
                continue

            # avoid any n-grams where a single gram is only punctuation
            grams = candidate.split()
            if not len([x for x in grams if TextUtils.is_punctuation(x)]):
                normalized.append(candidate)

        return normalized

    def _stopword_filter(self,
                         candidates: list) -> list:
        """ Stopword Filtering removes low-value phrases and words

        Architectural Decisions:
            https://github.com/craigtrim/dbpedia-tag/issues/3
            Net Net: We can't remove all low-value words; this has to be a consumer strategy

        Args:
            candidates (list): the incoming candidates

        Returns:
            list: the potentially filtered candidates
        """

        normalized = []
        for candidate in candidates:

            if candidate in stopwords:
                continue

            # avoid any n-grams where even a single gram is a stopwrd
            grams = candidate.split()
            if not len([x for x in grams if x in stopwords]):
                normalized.append(candidate)

        return normalized

    def _process(self,
                 candidates: list) -> list:

        candidates = self._stopword_filter(candidates)
        candidates = self._length_check(candidates)
        candidates = self._punkt_check(candidates)

        return candidates

    def process(self,
                candidates: list) -> list:
        sw = Stopwatch()

        results = self._process(candidates)

        if self.isEnabledForDebug:
            if results:
                Enforcer.is_list_of_str(results)

            removed = set(candidates).difference(set(results))

            self.logger.debug('\n'.join([
                "Sliding Window Blacklist Completed",
                f"\tRemoved Tokens: {list(removed)}",
                f"\tRetained Tokens: {results}",
                f"\tTotal Time: {str(sw)}"]))

        return results
